#include "dlPlaylist.h"


dlPlaylist::dlPlaylist(kmpPlaylist * destination, QString serverName, int serverPort, QString serverPassword, int stacksize)
    : QThread(stacksize)
{
    dest = destination;
    serverName_ = serverName;
    serverPort_ = serverPort;
    serverPassword_ = serverPassword;
}

void dlPlaylist::run()
    {
    //We create some variables to hold the songs we get, the enitys we get, and the new connection
    mpd_SongStore aSong;
    mpd_Connection * playlistConn;
    mpd_InfoEntity * entity;
    //We open a new connection at the current server and port for the playlist
    playlistConn = mpd_newConnection (serverName_, serverPort_, 10);
    checkForErrors(playlistConn);
    //Then we check to see if it opened fine
    if(serverPassword_ != "") {
        mpd_sendPasswordCommand(playlistConn, serverPassword_);
        checkForErrors(playlistConn);
        mpd_finishCommand(playlistConn);
        checkForErrors(playlistConn);
    }
    checkForErrors(playlistConn);
    //If we didn't open fine, we stop trying to use this connection, and return from the function
    if(playlistConn == NULL) return;
    //Now we reset the linked list and unallocate all of its currently stored memory
    ServerList.reset();
    //And to get the playlist we send this command! using -1 to get the WHOLE list
    mpd_sendPlaylistInfoCommand(playlistConn, -1);
    //We loop through each enitity that we get0
    while((entity = mpd_getNextInfoEntity(playlistConn))) {
        //If for some reason we don't get a song we just skip it.
        //  this should never happen but we do it anyways. b.s.t.s.
        if(entity->type!=MPD_INFO_ENTITY_TYPE_SONG) {
            mpd_freeInfoEntity(entity);
            continue;
        }
        //We set all of the information of the current song to a storable song type
        aSong.file = QString::fromUtf8(entity->info.song->file);
        aSong.artist = QString::fromUtf8(entity->info.song->artist);
        aSong.album = QString::fromUtf8(entity->info.song->album);
        aSong.title = QString::fromUtf8(entity->info.song->title);
        aSong.track = QString::fromUtf8(entity->info.song->track);
        aSong.time = entity->info.song->time;
        //And then copy it to the linked list
        ServerList.add(aSong);
        //this command releases the memory used to store this song
        mpd_freeInfoEntity(entity);
        //And check for errors
        checkForErrors(playlistConn);
        //If there was an error the connection would be set to NULL
        // so we exit the function if there was an error
        if(playlistConn==NULL) return;
    }
    //Now we send the end command, even though we are closing the connection
    // Its just proper to do this.
    
    mpd_finishCommand(playlistConn);
    //After we have gotten every song, its time to close the connection
    mpd_closeConnection(playlistConn);
    //And we need to tell the playlist window to update the viewed playlist with this new one
    dest->populatePlaylist(ServerList);
    //And we tell the user that the playlist has been updated, and at what time.
    
}

//Name:  checkForErrors
//When&What Calls:  this is called a BAZILLION TIMES
//Purpose:  To check to see if there is any errors in the connection
//  we call this ALOT of times, because it doesn't
//  hurt to check, and its VERY usefull to check,
//  as this stops the client from crashing!
void dlPlaylist::checkForErrors(mpd_Connection *& conn_)
{
    //If this connection does NOT exist, we try to reconnect
    if(conn_ == NULL) {
        return;
    }
    //If there IS an error then we do some checks
    if(conn_->error) {
        //We create a string variable to hold the error
        QString Error;
 //And get the error
 Error = QString(conn_->errorStr);
 if(conn_->error == MPD_ERROR_ACK) {
     //If the error is just an acknoledgement, (NON FATAL)
     // we just put it in the status bar for the user to see
     if(Error.contains("permission")) {
         QString myMessage = "\n\nYou must reconnect to the server with a different\n password if you wish to continue.\n\n";
         QMessageBox::warning( dest, "KMP", myMessage,"Ok", 0, 0, 0, 1 )  ;
     }
     //PASS ERROR TO CHILDREN&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
     //statusBar()->message(Error.right(Error.length()-4));
     //And clear the error so that everything will work correctly
     mpd_clearError(conn_);
     return;
 }
 //If we got here, then the error was fatal.... which is bad. So we disconnect.
 // gracefully of course :-p
 mpd_clearError(conn_);
 //And we close the connection
 //mpd_closeConnection(conn_);
 //And set it to NULL
 //dest->connError();
    }
}
